@@ -1,13 +1,14 @@
 '''TODO: Documentation
 '''
 
+from genericpath import isfile
 from bge import logic
 from bge.types import KX_GameObject as GameObject
 from mathutils import Vector
 from uplogic.audio import ULAudioSystem
 from uplogic.data import GlobalDB
 from uplogic.events import schedule_callback
-from uplogic.utils import interpolate
+from uplogic.utils import debug, interpolate
 import aud
 
 
@@ -94,7 +95,7 @@ class ULReverb():
             sample.distance_maximum = handle.distance_maximum
             sample.cone_angle_inner = handle.cone_angle_inner
             sample.pitch = handle.pitch
-            sample.volume = (1-(handle.volume * (mult**2)))*.5 * self.volume
+            sample.volume = (1-(handle.volume * (mult**2)))*.5 * self.volume * self.aud_system.volume
             sample.cone_volume_outer = handle.cone_volume_outer
 
 
@@ -116,11 +117,16 @@ class ULSound():
     def get_aud_sys(self, name: str):
         '''TODO: Documentation
         '''
+        scene = logic.getCurrentScene()
         aud_systems = GlobalDB.retrieve('uplogic.audio')
+        # print(scene.post_draw)
         if aud_systems.check(name):
-            return aud_systems.get(name)
+            aud_sys = aud_systems.get(name)
         else:
-            return ULAudioSystem(name)
+            aud_sys = ULAudioSystem(name)
+        if aud_sys.update not in scene.post_draw:
+            scene.post_draw.append(aud_sys.update)
+        return aud_sys
 
 
 class ULSound2D(ULSound):
@@ -136,18 +142,23 @@ class ULSound2D(ULSound):
         pitch: float = 1,
         loop_count: int = 1
     ):
+        self.finished = False
         if not (file and aud_system):
             return
         self.pitch = pitch
-        self.volume = volume
         self.aud_system = self.get_aud_sys(aud_system)
+        # print(self.aud_system)
+        self.volume = volume
         soundfile = logic.expandPath(file)
+        if not isfile(soundfile):
+            debug(f'Soundfile {soundfile} could not be loaded!')
+            return
         sound = aud.Sound(soundfile)
         device = self.aud_system.device
         handle = self.sound = device.play(sound)
         handle.relative = True
         handle.pitch = pitch
-        handle.volume = volume
+        handle.volume = volume * self.aud_system.volume
         handle.loop_count = loop_count
         self.aud_system.add(self)
 
@@ -160,7 +171,7 @@ class ULSound2D(ULSound):
             self.aud_system.remove(self)
             return
         handle.pitch = self.pitch * logic.getTimeScale()
-        handle.volume = self.volume
+        handle.volume = self.volume * self.aud_system.volume
 
 
 class ULSound3D(ULSound):
@@ -196,6 +207,7 @@ class ULSound3D(ULSound):
         cone_outer_volume: float = 0,
         aud_system: str = 'default'
     ):
+        self.finished = False
         if not (file and aud_system and speaker):
             return
         self._clear_sound = 1
@@ -204,14 +216,19 @@ class ULSound3D(ULSound):
         self.sounds = []
         self.reverb_samples = None
         self.aud_system = self.get_aud_sys(aud_system)
+        # print(self.aud_system)
         self.speaker = speaker
         self.reverb = reverb
         self.occlusion = occlusion
         self.volume = volume
         self.pitch = pitch
         self.cone_outer_volume = cone_outer_volume
+        master_volume = self.aud_system.volume
         self.transition = transition_speed
         soundfile = logic.expandPath(file)
+        if not isfile(soundfile):
+            debug(f'Soundfile {soundfile} could not be loaded!')
+            return
         sound = self.soundpath = aud.Sound(soundfile)
         device = self.aud_system.device
         handle = device.play(sound)
@@ -233,13 +250,13 @@ class ULSound3D(ULSound):
             handle.attenuation = attenuation
             handle.orientation = speaker.worldOrientation.to_quaternion()
             handle.pitch = pitch
-            handle.volume = volume
+            handle.volume = volume * master_volume
             handle.distance_reference = distance_ref
             handle.distance_maximum = 1000
             handle.cone_angle_inner = cone_angle[0]
             handle.cone_angle_outer = cone_angle[1]
             handle.loop_count = loop_count
-            handle.cone_volume_outer = cone_outer_volume * volume
+            handle.cone_volume_outer = cone_outer_volume * volume * master_volume
         if self.reverb:
             self.reverb_samples = ULReverb(
                 self,
@@ -322,11 +339,13 @@ class ULSound3D(ULSound):
                     if not i
                     else (1 - cs) * sustained
                 )
-                handle.volume = self.volume * mult
+                master_volume = self.aud_system.volume
+                handle.volume = self.volume * mult * master_volume
                 handle.cone_volume_outer = (
                     self.cone_outer_volume *
                     self.volume *
-                    mult
+                    mult *
+                    master_volume
                 )
         if self.reverb_samples:
             self.reverb_samples.update()
